@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.6.2.17 2009/11/29 08:29:14 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.6.2.21 2010/01/21 05:49:10 t-ishii Exp $
  * 
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -346,11 +346,12 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			return POOL_CONTINUE;
 		}
 
-		if (IsA(node, PrepareStmt) || IsA(node, DeallocateStmt) ||
+		if (IsA(node, PrepareStmt) ||
 			IsA(node, VariableSetStmt) || IsA(node, DiscardStmt))
 		{
 			/*
-			 * PREPARE, DEALLOCATE and SET statements must be replicated.
+			 * PREPARE, SET and DISCARD statements must be
+			 * replicated even if we are in master/slave mode.
 			 */
 			if (MASTER_SLAVE && TSTATE(backend) != 'E')
 				force_replication = 1;
@@ -911,7 +912,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 	len = ntohl(len) - 4;
 	string = pool_read2(frontend, len);
 
-	pool_debug("Parse: portal name <%s>", string);
+	pool_debug("Parse: statement name <%s>", string);
 
 	name = string;
 	stmt = string + strlen(string) + 1;
@@ -963,6 +964,11 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		/* translate Parse message to PrepareStmt */
 		p_stmt = palloc(sizeof(PrepareStmt));
 		p_stmt->type = T_PrepareStmt;
+
+		/* XXX: there's a confusion here. Someone mixed up statement
+		 * name with portal name. It is regarded that statment name ==
+		 * portal name. Someday we should fix this. Sigh.
+		 */
 		p_stmt->name = pstrdup(name);
 		p_stmt->query = copyObject(node);
 		portal->stmt = (Node *)p_stmt;
@@ -1560,7 +1566,9 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			/* fall through */
 
 		default:
-			if (MAJOR(backend) == PROTO_MAJOR_V3)
+			if ((MAJOR(backend) == PROTO_MAJOR_V3) &&
+			    (fkind == 'S' || fkind == 'H' || fkind == 'D' || fkind == 'f'||
+				 fkind == 'C' || fkind == 'B' || fkind == 'F' || fkind == 'd' || fkind == 'c'))
 			{
 				if (MASTER_SLAVE &&
 					(TSTATE(backend) != 'I' || receive_extended_begin))
@@ -2519,7 +2527,7 @@ static int is_temp_table(POOL_CONNECTION_POOL *backend, Node *node)
 		}
 	}
 
-	hasrelistemp = (int)pool_search_relcache(hasrelistemp_cache, backend, "pg_class");
+	hasrelistemp = pool_search_relcache(hasrelistemp_cache, backend, "pg_class")==0?0:1;
 	if (hasrelistemp)
 		query = ISTEMPQUERY84;
 	else
@@ -2543,6 +2551,6 @@ static int is_temp_table(POOL_CONNECTION_POOL *backend, Node *node)
 	/*
 	 * Search relcache.
 	 */
-	result = (int)pool_search_relcache(relcache, backend, str);
+	result = pool_search_relcache(relcache, backend, str)==0?0:1;
 	return result;
 }
